//+------------------------------------------------------------------+
//|                                                     BGL_FUND.mq4 |
//|                        Copyright 2019, MetaQuotes Software Corp. |
//|                                             https://www.mql5.com |
//+------------------------------------------------------------------+
#property copyright "Copyright 2019, MetaQuotes Software Corp."
#property link      "https://www.mql5.com"
#property version   "1.00"
#property strict

#include <Arrays\ArrayString.mqh>
#include <Charts\Chart.mqh>

#define BTN_HEIGHT                        (20)
#define BTN_WIDTH                         (100)
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
enum ValueOfSpread
  {
   spread_no,//No
   spread_b05,// > 0.05 %

  };
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
enum StartHour
  {
   hour_any,//Any time
   hour_9am,// 9 am
   hour_10am,// 10 am 
   hour_4pm,// 4 pm 
   hour_0am,// Midnight
  };

sinput string        delimeter_01=""; // --- Filtration settings ---
input bool           noSYMBmarketWath=true; // Hide if not in the Market Watch
input bool           noSYMBwithPOS=true; // Hide if there are positions
input ValueOfSpread  hide_SPREAD=spread_b05; // Hide in case of a spread
input uint           hide_PRICE_HIGH=0; // Hide if the price is higher
input uint           hide_PRICE_LOW=0; // Hide if the price is lower
input bool           hideProhibites=true; // Hide if trading is disabled
input bool           hideClosed=true; // Hide if the market is closed
input StartHour      hide_HOURS=hour_any; // Show if there is an open time
input double         hideATRcents=0.0050; // Hide if ATR is less than the set value in dollars
sinput string        delimeter_02=""; // --- Chart settings ---
input bool           addInfoWatch=false; // Add the chart to the Market Watch
input bool           viewCandle=true; // Open candlestick charts
input bool           viewVolumes=false; // Show tick volumes
input bool           showInfoSymbol=true; // Show movement direction
input bool           showNameSymbol=true; // Show symbol name


                                          // prefix to be added to the names of all graphical objects created by the EA:
string exprefix="finder";
// array of symbols fitting our conditions:
CArrayString arrPanel1;
// index of the current symbol in the arrPanel1 array:
int panel1val;
// array that is to store charts created by the EA (there is only one chart here at the moment):
CChart charts[];
// array that is to store pointers to charts created by the EA (there is only one pointer here at the moment):
long curChartID[];
int levels_count;
//+------------------------------------------------------------------+
//| Expert initialization function                                   |
//+------------------------------------------------------------------+
int OnInit()
  {
   start_symbols();

//--- create timer
   EventSetTimer(1);

//---
   return(INIT_SUCCEEDED);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
void start_symbols()
  {
   levels_count=4;
// set the index of the current symbol in the list to zero (the first symbol in the array):
   panel1val=0;
// prepare the symbol list:
   prepare_symbols();
// remove previously created symbol buttons from the chart:
   ObjectsDeleteAll(0,exprefix);
// display symbol list:
   show_symbols();
// update the chart to see changes:
   ChartRedraw(0);
  }
//+------------------------------------------------------------------+
//| Expert deinitialization function                                 |
//+------------------------------------------------------------------+
void OnDeinit(const int reason)
  {
   if(reason!=REASON_CHARTCHANGE)
     {
      ObjectsDeleteAll(0,exprefix);
     }
   EventKillTimer();
  }
//+------------------------------------------------------------------+
//| Expert tick function                                             |
//+------------------------------------------------------------------+
void OnTick()
  {
//---

  }
//+------------------------------------------------------------------+
//| Timer function                                                   |
//+------------------------------------------------------------------+
void OnTimer()
  {
// if the open charts ID array contains values, then...
   uchar tmpCIDcnt=(uchar) ArraySize(curChartID);
   if(tmpCIDcnt>0)
     {
      // if the last ID in the array is not corrupted, then...
      if(curChartID[tmpCIDcnt-1]>0)
        {
         // if the chart with the ID has no buttons, create them
         if(ObjectFind(curChartID[tmpCIDcnt-1],exprefix+"_p_btn_next")<0)
           {
            createBTNS(curChartID[tmpCIDcnt-1]);
              }else{
            if(ObjectGetInteger(curChartID[tmpCIDcnt-1],exprefix+"_p_btn_prev",OBJPROP_STATE)==true)
              {
               prevchart();
               return;
              }
            if(ObjectGetInteger(curChartID[tmpCIDcnt-1],exprefix+"_p_btn_next",OBJPROP_STATE)==true)
              {
               nextchart();
               return;
              }
            if(ObjectGetInteger(curChartID[tmpCIDcnt-1],exprefix+"_p_btn_close",OBJPROP_STATE)==true)
              {
               closecharts();
               return;
              }
           }
        }
     }

  }
//+------------------------------------------------------------------+

void nextchart()
  {
   levels_count=4;

// if a symbol list features the next symbol, open its chart
// otherwise, close the current chart
   if(arrPanel1.Total()>(panel1val+1))
     {
      panel1val++;
      showcharts(arrPanel1[panel1val]);
        }else{
      closecharts();
     }
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
void prevchart()
  {
   levels_count=4;
// if a symbol list features the previous symbol, open its chart
// otherwise, close the current chart
   if(arrPanel1.Total()>(panel1val-1) && (panel1val-1)>=0)
     {
      panel1val--;
      showcharts(arrPanel1[panel1val]);
        }else{
      closecharts();
     }
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
void curchart()
  {
   if(arrPanel1.Total()>panel1val)
     {
      showcharts(arrPanel1[panel1val]);
        }else{
      closecharts();
     }
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
void closecharts()
  {
// if the array of charts opened by the EA is not empty, then...
   if(ArraySize(charts))
     {
      // close all the charts consistently
      for(int i=0; i<ArraySize(charts); i++)
        {
         charts[i].Close();
        }
      // clear the array of charts
      ArrayFree(charts);
     }
// if the array of IDs of the chart opened by the EA is not empty, clear it
   if(ArraySize(curChartID))
     {
      ArrayFree(curChartID);
     }
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
void showcharts(string name)
  {
// if charts were already open, close them
   closecharts();

// Add a symbol to the "Market Watch" panel if it is not there
// and if the "Add chart to Market Watch" input is 'true'
   if(addInfoWatch)
     {
      SymbolSelect(name,true);
     }

// open the chart and place the chart ID to the curChartID array
   curChartID[ArrayResize(curChartID,ArraySize(curChartID)+1)-1]=charts[(uchar) ArrayResize(charts,ArraySize(charts)+1)-1].Open(name,PERIOD_D1);

// if the "Open candlestick charts" input is 'true',
// switch the chart to the candlestick mode
   if(viewCandle)
     {
      ChartSetInteger(curChartID[ArraySize(curChartID)-1],CHART_MODE,CHART_CANDLES);
     }
// if the "Show tick volumes" input is 'true',
// display the tick volumes
   if(viewVolumes)
     {
      ChartSetInteger(curChartID[ArraySize(curChartID)-1],CHART_SHOW_VOLUMES,CHART_VOLUME_TICK);
     }
// change the chart scale to the most convenient one
   ChartSetInteger(curChartID[ArraySize(curChartID)-1],CHART_SCALE,2);

//display additional info on the chart
   string msg="";
   if(showNameSymbol)
     {
      StringAdd(msg,getmename_symbol(name)+"\r\n");
     }
   if(showInfoSymbol)
     {
      StringAdd(msg,getmeinfo_symbol(name,false)+"\r\n");
     }
   if(StringLen(msg)>0)
     {
      ChartSetString(curChartID[ArraySize(curChartID)-1],CHART_COMMENT,msg);
     }

// wait for one third of a second till all the changes are implemented
   Sleep(333);
// update the open chart to make all the changes
   ChartRedraw(curChartID[ArraySize(curChartID)-1]);

  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
string getmename_symbol(string symname)
  {
   return SymbolInfoString(symname, SYMBOL_DESCRIPTION);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
string getmeinfo_symbol(string symname,bool show=true)
  {
   MqlRates rates2[];
   ArraySetAsSeries(rates2,true);
   string msg="";

   if(CopyRates(symname,PERIOD_D1,0,1,rates2)>0)
     {
      if(show)
        {
         StringAdd(msg,(string) symname+": ");
        }
      StringAdd(msg,"D1 ");
      if(rates2[0].close>rates2[0].open)
        {
         StringAdd(msg,"+"+DoubleToString(((rates2[0].close-rates2[0].open)/rates2[0].close)*100,2)+"%");
           }else{
         if(rates2[0].close<rates2[0].open)
           {
            StringAdd(msg,"-"+DoubleToString(((rates2[0].open-rates2[0].close)/rates2[0].close)*100,2)+"%");
              }else{
            StringAdd(msg,"0%");
           }
        }
     }
   if(CopyRates(symname,PERIOD_H1,0,1,rates2)>0)
     {
      StringAdd(msg,", H1 ");
      if(rates2[0].close>rates2[0].open)
        {
         StringAdd(msg,"+"+DoubleToString(((rates2[0].close-rates2[0].open)/rates2[0].close)*100,2)+"% (+"+DoubleToString(rates2[0].close-rates2[0].open,(int) SymbolInfoInteger(symname,SYMBOL_DIGITS))+" "+SymbolInfoString(symname,SYMBOL_CURRENCY_PROFIT)+")");
           }else{
         if(rates2[0].close<rates2[0].open)
           {
            StringAdd(msg,"-"+DoubleToString(((rates2[0].open-rates2[0].close)/rates2[0].close)*100,2)+"% (-"+DoubleToString(rates2[0].open-rates2[0].close,(int) SymbolInfoInteger(symname,SYMBOL_DIGITS))+" "+SymbolInfoString(symname,SYMBOL_CURRENCY_PROFIT)+")");
              }else{
            StringAdd(msg,"0%");
           }
        }
     }

   return msg;
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
void createBTNS(long CID)
  {
   string name=ChartSymbol(CID);

   ObjectCreate(CID,exprefix+"_p_btn_prev",OBJ_BUTTON,0,0,0);
   ObjectSetInteger(CID,exprefix+"_p_btn_prev",OBJPROP_XDISTANCE,110);
   ObjectSetInteger(CID,exprefix+"_p_btn_prev",OBJPROP_YDISTANCE,90);
   ObjectSetInteger(CID,exprefix+"_p_btn_prev",OBJPROP_XSIZE,BTN_WIDTH);
   ObjectSetInteger(CID,exprefix+"_p_btn_prev",OBJPROP_YSIZE,BTN_HEIGHT);
   ObjectSetInteger(CID,exprefix+"_p_btn_prev",OBJPROP_CORNER,CORNER_LEFT_LOWER);
   ObjectSetString(CID,exprefix+"_p_btn_prev",OBJPROP_TEXT,"Prev chart");
   ObjectSetInteger(CID,exprefix+"_p_btn_prev",OBJPROP_SELECTABLE,false);

   ObjectCreate(CID,exprefix+"_p_btn_next",OBJ_BUTTON,0,0,0);
   ObjectSetInteger(CID,exprefix+"_p_btn_next",OBJPROP_XDISTANCE,110);
   ObjectSetInteger(CID,exprefix+"_p_btn_next",OBJPROP_YDISTANCE,65);
   ObjectSetInteger(CID,exprefix+"_p_btn_next",OBJPROP_XSIZE,BTN_WIDTH);
   ObjectSetInteger(CID,exprefix+"_p_btn_next",OBJPROP_YSIZE,BTN_HEIGHT);
   ObjectSetInteger(CID,exprefix+"_p_btn_next",OBJPROP_CORNER,CORNER_LEFT_LOWER);
   ObjectSetString(CID,exprefix+"_p_btn_next",OBJPROP_TEXT,"Next chart");
   ObjectSetInteger(CID,exprefix+"_p_btn_next",OBJPROP_SELECTABLE,false);

   ObjectCreate(CID,exprefix+"_p_btn_close",OBJ_BUTTON,0,0,0);
   ObjectSetInteger(CID,exprefix+"_p_btn_close",OBJPROP_XDISTANCE,110);
   ObjectSetInteger(CID,exprefix+"_p_btn_close",OBJPROP_YDISTANCE,40);
   ObjectSetInteger(CID,exprefix+"_p_btn_close",OBJPROP_XSIZE,BTN_WIDTH);
   ObjectSetInteger(CID,exprefix+"_p_btn_close",OBJPROP_YSIZE,BTN_HEIGHT);
   ObjectSetInteger(CID,exprefix+"_p_btn_close",OBJPROP_CORNER,CORNER_LEFT_LOWER);
   ObjectSetString(CID,exprefix+"_p_btn_close",OBJPROP_TEXT,"Close chart");
   ObjectSetInteger(CID,exprefix+"_p_btn_close",OBJPROP_SELECTABLE,false);

// update the chart to see the implemented changes
   ChartRedraw(CID);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
void OnChartEvent(const int id,         // event ID   
                  const long& lparam,   // event parameter of the long type 
                  const double& dparam, // event parameter of the double type 
                  const string& sparam) // event parameter of the string type 
  {
   switch(id)
     {
      case CHARTEVENT_OBJECT_CLICK:
         // if the button name contains a line present in all graphical objects
         // created by your EA, then...
         if(StringFind(sparam,exprefix+"btn")>=0)
           {
            // place the current button index
            // (position of a current symbol in the symbol list) to the panel1val variable
            string tmpme=sparam;
            StringReplace(tmpme,exprefix+"btn","");
            panel1val=(int) tmpme;

            // open the current symbol chart
            showcharts(ObjectGetString(0,sparam,OBJPROP_TEXT));
           }
         break;
      case CHARTEVENT_KEYDOWN:
         switch((int) sparam)
           {
            case 45: //x
               ExpertRemove();
               break;
            case 19: //r
               start_symbols();
               break;
           }
         break;
     }

  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
void show_symbols()
  {

// initialize the variables for defining the X and Y coordinates
   int btn_left=0;
   int btn_line=1;
   int btn_right=(int) ChartGetInteger(0,CHART_WIDTH_IN_PIXELS)-77;

// display the button on the chart for each symbol in the array
// write a symbol name on the button
   for(int i=0; i<arrPanel1.Total(); i++)
     {
      if(btn_left>btn_right-BTN_WIDTH)
        {
         btn_line++;
         btn_left=0;
        }

      ObjectCreate(0,exprefix+"btn"+(string) i,OBJ_BUTTON,0,0,0);
      ObjectSetInteger(0,exprefix+"btn"+(string) i,OBJPROP_XDISTANCE,btn_left);
      ObjectSetInteger(0,exprefix+"btn"+(string) i,OBJPROP_YDISTANCE,BTN_HEIGHT*btn_line);
      ObjectSetInteger(0,exprefix+"btn"+(string) i,OBJPROP_XSIZE,BTN_WIDTH);
      ObjectSetInteger(0,exprefix+"btn"+(string) i,OBJPROP_YSIZE,BTN_HEIGHT);
      ObjectSetInteger(0,exprefix+"btn"+(string) i,OBJPROP_FONTSIZE,8);
      ObjectSetInteger(0,exprefix+"btn"+(string) i,OBJPROP_COLOR,clrBlack);
      ObjectSetString(0,exprefix+"btn"+(string) i,OBJPROP_TEXT,arrPanel1.At(i));
      ObjectSetInteger(0,exprefix+"btn"+(string) i,OBJPROP_SELECTABLE,false);

      btn_left+=BTN_WIDTH;
     }

  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
void prepare_symbols()
  {
// variable for temporary storage of a symbol name
   string name;
// variable for storing the last symbol quotes
   MqlTick lastme;

// reset the symbol array if it contains any values
   arrPanel1.Resize(0);

// form the temporary tmpSymbols array
// it will contain all available symbols
   CArrayString tmpSymbols;
   for(int i=0; i<SymbolsTotal(noSYMBmarketWath); i++)
     {
      tmpSymbols.Add(SymbolName(i,noSYMBmarketWath));
     }

// conditions are to be checked here
// and a symbol is to be included into the symbol list
// if it fits the conditions
   for(int i=0; i<tmpSymbols.Total(); i++)
     {
      name=tmpSymbols[i];

      // remove excessive spaces from a symbol name,
      // since we do not know for sure where it came from
      StringTrimLeft(name);
      StringTrimRight(name);
      if(!StringLen(name))
        {
         continue;
        }

      // main filtration of symbols is performed further on

      // Hide symbols unavailable for trading
      if(hideProhibites)
        {
         // skip if the minimum position volume on a symbol is 0
         if(SymbolInfoDouble(name,SYMBOL_VOLUME_MIN)==0) continue;
         // skip if opening positions is disabled on a symbol
         if(SymbolInfoInteger(name,SYMBOL_TRADE_MODE)==SYMBOL_TRADE_MODE_DISABLED || SymbolInfoInteger(name,SYMBOL_TRADE_MODE)==SYMBOL_TRADE_MODE_CLOSEONLY)
           {
            continue;
           }
        }

      // Hide symbols having positions
      bool isskip=false;
      if(noSYMBwithPOS)
        {
         // view the list of all open positions
         int cntMyPos=OrdersTotal();
         for(int ti=cntMyPos-1; ti>=0; ti--)
           {
            if(OrderSelect(ti,SELECT_BY_POS,MODE_TRADES)==false) continue;
            if(OrderSymbol()==name)
              {
               isskip=true;
               break;
              }
           }
        }
      if(isskip)
        {
         continue;
        }

      MqlDateTime curDay;
      TimeCurrent(curDay);
      MqlDateTime curDayFrom;
      datetime dfrom;
      datetime dto;

      bool sessionData=SymbolInfoSessionTrade(name,(ENUM_DAY_OF_WEEK) curDay.day_of_week,0,dfrom,dto);

      // hide a symbol if the market is closed today
      if(hideClosed && !sessionData)
        {
         continue;
        }

      // Show only symbols opening in
      // get the current day to the curDay variable
      // if there is a limitation on market open time, 
      // and we managed to get open time of the current stock for the current day, then...
      if(hide_HOURS!=hour_any && sessionData)
        {
         TimeToStruct(dfrom,curDayFrom);
         if(hide_HOURS==hour_9am && curDayFrom.hour!=9)
           {
            continue;
           }
         if(hide_HOURS==hour_10am && curDayFrom.hour!=10)
           {
            continue;
           }
         if(hide_HOURS==hour_4pm && curDayFrom.hour!=16)
           {
            continue;
           }
         if(hide_HOURS==hour_0am && curDayFrom.hour!=0)
           {
            continue;
           }
        }

      // if the inputs using the value of the symbol's current price are active,
      // attempt to get the current values
      if(hide_PRICE_HIGH>0 || hide_PRICE_LOW>0 || hide_SPREAD>0)
        {
         SymbolInfoTick(name,lastme);
         if(lastme.bid==0)
           {
            Alert("Failed to get BID value. Some filtration functions may not work.");
           }
        }
      // Hide symbols with a spread:
      if(hide_SPREAD>0 && lastme.bid>0)
        {
         switch(hide_SPREAD)
           {
            // if the current spread exceeds 0.05% of the price, skip the symbol
            case spread_b05:
               if(((SymbolInfoInteger(name,SYMBOL_SPREAD)*SymbolInfoDouble(name,SYMBOL_POINT))/lastme.bid)*100>0.05)
                 {
                  isskip=true;
                 }

               break;
           }
        }
      if(isskip)
        {
         continue;
        }

      // Hide symbols with the higher price (0 - do not hide)
      if(hide_PRICE_HIGH>0 && lastme.bid>0 && lastme.bid>hide_PRICE_HIGH)
        {
         continue;
        }

      // Hide symbols with the lower price (0 - do not hide)
      if(hide_PRICE_LOW>0 && lastme.bid>0 && lastme.bid<hide_PRICE_LOW)
        {
         continue;
        }

      // Hide if ATR is less than the set value in dollars
      if(hideATRcents>0)
        {
         MqlRates rates[];
         ArraySetAsSeries(rates,true);
         double atr;
         if(CopyRates(name,PERIOD_D1,1,5,rates)==5)
           {
            atr=0;
            for(int j=0; j<5; j++)
              {
               atr+=rates[j].high-rates[j].low;
              }
            atr/=5;
            if(atr>0 && atr<hideATRcents)
              {
               continue;
              }
           }
        }

      // if a symbol fits all our conditions, add it to the list
      arrPanel1.Add(name);
     }
  }
//+------------------------------------------------------------------+
